import { action, observable, autorun } from 'mobx';

const fn = action(function (a, b, c) {
  // 状態を変更する処理
  console.log(a, b, c);
});
fn('a', 'b', 'c');

// 第1引数に関数名を指定可能
const fn2 = action('someFunction', function (a, b, c) {
  // 状態を変更する処理
  console.log(a, b, c);
});
fn2('a', 'b', 'c');

console.log('');
console.log('@action ----------------------');
console.log('');

class State {
  @observable count = 0;
  // アクションの定義
  @action countUp() {
    this.count += 1;
  }
  // アクション名を指定
  @action('countDown') countDown() {
    this.count -= 1;
  }
}
const state = new State();
autorun(() => console.log('state.count', state.count));
state.countUp();
state.countUp();
state.countDown();

console.log('');
console.log('@action.bound ----------------------');
console.log('');

class StateWithBound {
  @observable count = 0;
  // 実行時のthisが必ずクラスのインスタンスであることを保証
  @action.bound countUp() {
    this.count += 1;
  }
}
const state2 = new StateWithBound();
autorun(() => console.log('state2.count', state2.count));
// bind関数を使われても正しく動作する
state2.countUp.bind(null)();