import * as mobx from 'mobx';
const { observable, autorun, action, runInAction } = mobx;

mobx.useStrict(true);

const state = observable({
  data: null
});
autorun(() => console.log('state.data', state.data));

// Promiseを返すダミーの非同期処理
const fetchServerData = () =>
  new Promise(resolve => resolve({ key: 'value' }));

console.log('callback with action -----------');

const asyncAction1 = action(function () {
  fetchServerData().then(
    // コールバック関数もアクションにする
    action('suceededCallback', function (data) {
      state.data = data;
    })
  )
});
asyncAction1();

console.log('callback with runInAction -----------');

const asyncAction2 = action(function () {
  fetchServerData().then(data => {
    // runInAction関数: 即時実行されるアクションを定義
    runInAction(() => {
      state.data = data;
    });
  });
});
asyncAction2();

console.log('async/await & runInAction -----------');

// async/awaitとrunInAction関数を使って同期的っぽく記述する
const asyncAction3 = action(async function () {
  const data = await fetchServerData();
  // runInAction関数: 即時実行されるアクションを定義
  runInAction(() => {
    state.data = data;
  });
});
asyncAction3();