import React from 'react';
import ReactDOM from 'react-dom';
import { useStrict, observable, action } from 'mobx';
import { observer, Provider, inject } from 'mobx-react';

// strictモードを有効化
useStrict(true);
// 状態とカウントアップするアクションを定義
// 1. 状態とアクションをまとめたクラスを定義
class State {
  @observable count = 0;
  @action.bound countUp() { this.count += 1; }
}

// 3. 紐つけたインスタンスをコンポーネントに紐付け
@inject('state')
@observer
class CounterApp extends React.Component {
  render() {
    // 4. 属性経由でインスタンスを受け取る
    const { state } = this.props;
    return (
      <div>
        <h1>count: {state.count}</h1>
        <button onClick={state.countUp}>+1</button>
      </div>
    );
  }
}
ReactDOM.render(
  // 2. stateという名前にインスタンスを紐付け
  <Provider state={new State()}>
    <CounterApp />
  </Provider>,
  document.getElementById('root')
);